# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from os_faults.ansible import executor


class FakeAnsibleResult(object):

    def __init__(self, payload, host=None, status=executor.STATUS_OK):
        self.host = host
        self.payload = payload
        self.status = status

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented
        if self.host != other.host:
            return False
        if self.payload != other.payload:
            return False
        if self.status != other.status:
            return False
        return True

    def __ne__(self, other):
        return not self == other
